function w = AMAengineTest(nf,bPLOT)
% function w = AMAengineTest(nf,bPLOT)
%
%   example calls: % TO LEARN 1 FILTER ON TEST DATA
%                    w = AMAengineTest;
%                  % TO LEARN 2 FILTER ON TEST DATA
%                    w = AMAengineTest(2);
%
% test performance of AMAengine 
%
% nf:    number of filters to learn (default = 1)
% bPLOT: 1 -> plot, 0 -> not        (default = 1)  
% %%%%%%%%%%%%%%%%%%%%%%%%%%
% w:     AMA filters [numDimensions x numFilters]

% INPUT HANDLING
if ~exist('nf','var')
    nf = 1;
end
if ~exist('bPLOT','var')
    bPLOT = 1;
end

% LOAD TEST DATA
load('AMAengineTestData.mat'); % s:      test stims       [numStims x numDimensions]
                               % ctgInd: category indices [numStims x 1]

% GENERATE RANDOM FILTER WEIGHTS
w0 = randn(size(s,2),nf);
w0 = bsxfun(@rdivide,w0,sqrt(sum(w0.^2,1)));

% SET MINIMIZATION FUNCTION OPTIONS
opts             = optimset('fmincon');
opts.Display     = 'iter';

% MINIMIZE OBJECTIVE FUNCTION
[w] = fmincon(@(w) objFunc(w,s,ctgInd),w0,[],[],[],[],[],[],@(w) wConEq(w),opts);

% PLOT RESULTS
if bPLOT
    disp(['AMAengineTest: WARNING! plotting not functional unless numDims perfect square']);
    %%%%%%%%%%%%%%%%%%%%%
    % SHOW STIMULUS SET %
    %%%%%%%%%%%%%%%%%%%%%
    % PLOT A STIM FROM CTG 1
    figure('position',[893   853   447   253]); 
    subplot(1,2,1);
    imagesc(reshape(mean(s(ctgInd == 1,:),1),sqrt(size(s,2))*[1 1]));
    title('Ctg 1 Avg Stim','fontsize',18);
    colormap jet;
    axis square;
    caxis(max(abs(w(:))).*[-1 1]);
    set(gca,'xtick',[]);
    set(gca,'ytick',[]);
    
    % PLOT A STIM FROM CTG 2
    subplot(1,2,2);
    imagesc(reshape(mean(s(ctgInd == 2,:),1),sqrt(size(s,2))*[1 1]));
    title('Ctg 2 Avg Stim','fontsize',18);
    colormap jet;
    axis square;
    caxis(max(abs(w(:))).*[-1 1]);
    cb=colorbar;
    set(cb,'position',[0.9172    0.2197    0.0203    0.6000]);
    set(gca,'xtick',[]);
    set(gca,'ytick',[]);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % SHOW INITIAL & AMA FILTERS %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i = 1:size(w,2) % loop over filters
        figure('position',[893   580   446   250]); 
        % PLOT INITIAL FILTER
        subplot(1,2,1);
        imagesc(reshape(w0(:,i),sqrt(size(s,2)).*[1 1]));
        title(['Init F' num2str(i)],'fontsize',18);
        caxis(max(abs(w(:))).*[-1 1]);
        set(gca,'xtick',[]);
        set(gca,'ytick',[]);
        axis square;
        
        % PLOT AMA FILTER
        subplot(1,2,2);
        imagesc(reshape(w(:,i),sqrt(size(s,2)).*[1 1]));
        title(['AMA F' num2str(i)],'fontsize',18);
        caxis(max(abs(w(:))).*[-1 1]);
        cb=colorbar;
        set(cb,'position',[0.9172    0.2197    0.0203    0.6000]);
        set(gca,'xtick',[]);
        set(gca,'ytick',[]);
        axis square;
    end
    
    % PLOT STIMULUS RESPONSE DISTRIBUTIONS
    figure('position',[892   301   447   228]);
    % COMPUTE (NORMALIZED) FILTER RESPONSES BY CATEGORY
    r = s*w; % ctg 1 filter responses
    if size(w,2) == 1
        % COMPUTE FILTER RESPONSE HISTOGRAMS
        rSmp = -1:.05:1; 
        r1hist = hist(r(ctgInd==1,:),rSmp); 
        r2hist = hist(r(ctgInd==2,:),rSmp);
        % PLOT THEM
        plot(rSmp,r1hist,'bo-',rSmp,r2hist,'rs-','linewidth',2)
        xlabel('Filter Response','fontsize',15);
        ylabel('Frequency','fontsize',15);
    elseif size(w,2) > 1
        if size(w,2) > 2
           disp(['AMAengineTest: WARNING! plotting filter responses for 1st two filters only']); 
        end
        plot(r(ctgInd==1,1),r(ctgInd==1,2),'bo',r(ctgInd==2,1),r(ctgInd==2,2),'rs');
        xlabel('F1 Response','fontsize',15);
        ylabel('F2 Response','fontsize',15);
        axis square;
    end
    legend('Ctg1','Ctg2',-1);
end


%%% OBJECTIVE FUNCTION %%%
function E = objFunc(w,s,ctgInd)
% function E = objFunc(s,w,ctgInd)
% 
% objective function for AMA
% 
% s:      stimuli. vector magnitude of each stimulus must equal 1  [numStims      x numDimensions]       
% w:      filters. vector magnitude of each  filter  must equal 1  [numDimensions x numFilters]          
% ctgInd: category membership of each stimulus                     [numDimensions x 1]               
% %%%%%%
% E:      objective to minimize
r      =  stim2resp(s,w,5.7);            % mean response from filter weights and stimuli
sigma  =  resp2sigma(r,1.36,0.23);       % sigma from mean response
pp     =  AMAengine(r,sigma,ctgInd);     % posterior probability of correct category
E      =  sum(-(1/size(s,1)).*log2(pp)); % average relative entropy in bits

%%% CONSTRAINT FUNCTION %%%
function [c,ceq] = wConEq(w)
c = []; 
ceq = sum(w.^2)-1;                  % filter vector magnitude must equal 1

%%%  RESPONSE FUNCTION  %%%
function [r] = stim2resp(s,w,rMax)
r = rMax.*(s*w);

%%%   SIGMA   FUNCTION  %%% 
function [sigma] = resp2sigma(r,fano,varBase)
sigma = sqrt( fano.*abs(r) + varBase );
